import 'package:flutter_gadgets/flutter_gadgets.dart';

import '../model/game.dart';
import '../view/game_screen.dart';

void onResetPressed() {
  final model = ServiceContainer.instance.observableModel;
  final game = model.getBean<Game>();
  game.newGame();
  final viewModel = model.getBean<GameScreenVM>();
  viewModel.controller.clear();
  viewModel.errorMessage = null;
  viewModel.canPlay = true;
  model.notifyFor(GameScreenVM);
  model.notifyFor(Game);
}

void onNextTurnPressed() {
  final model = ServiceContainer.instance.observableModel;
  final viewModel = model.getBean<GameScreenVM>();
  final value = viewModel.controller.text.trim();
  if (value.isEmpty) {
    viewModel.errorMessage = 'You must insert a number.';
    model.notifyFor(GameScreenVM);
    return;
  }
  final parsedValue = int.tryParse(value);
  if (parsedValue == null) {
    viewModel.errorMessage = 'You must insert a valid number.';
    model.notifyFor(GameScreenVM);
    return;
  } else if (parsedValue < 1 || parsedValue > 100) {
    viewModel.errorMessage = 'The value must be between 1 and 100.';
    model.notifyFor(GameScreenVM);
    return;
  } else {
    final game = model.getBean<Game>();
    game.nextTurn(parsedValue);
    viewModel.errorMessage = null;
    if (game.numberFound) {
      viewModel.controller.clear();
      viewModel.canPlay = false;
    }
    model.notifyFor(GameScreenVM);
    model.notifyFor(Game);
  }
}
