import 'package:flutter/material.dart';

/// An [InheritedModel] that is used for updating [SubscriberGadget]s.
/// Each property represents a bean or an aspect (property) of a bean.
/// When properties are updated, observers will be rebuilt.
///
/// NOTE: Do not use this widget manually!
class ModelGadget extends InheritedModel<String> {
  ModelGadget({Key key, @required this.updatedProperties, @required this.child})
      : assert(child != null),
        super(key: key, child: child);

  /// The widget tree provided by [AppGadget].
  final Widget child;

  /// A set of updated properties representing a bean or an aspect (property) of a bean that changed.
  final Set<String> updatedProperties;

  @override
  bool updateShouldNotify(InheritedWidget oldWidget) => true;

  @override
  bool updateShouldNotifyDependent(
      InheritedModel<String> oldWidget, Set<String> dependencies) {
    if (dependencies == null ||
        dependencies.isEmpty ||
        updatedProperties == null ||
        updatedProperties.isEmpty) return false;
    return dependencies.intersection(updatedProperties).isNotEmpty;
  }
}
