import 'package:flutter_gadgets/flutter_gadgets.dart';
import 'package:flutter_test/flutter_test.dart';

import '../utils.dart';

void main() {
  group('ObservableModel', () {
    final token = 'token';
    final observableModel = ObservableModel();
    final mockBean = MockBean();
    final mockBeanWithToken = MockBean();
    observableModel.putBean<MockBean>(mockBean, notify: false);
    observableModel.putBean<MockBean>(mockBeanWithToken, token: token, notify: false);
    test('toBeanKey: bean only', () {
      expect(toBeanKey(MockBean), 'MockBean__');
    });
    test('toBeanKey: bean + token', () {
      expect(toBeanKey(MockBean, token: 'token'), 'MockBean_token_');
    });
    test('toBeanKey: bean + property', () {
      expect(toBeanKey(MockBean, property: 'property'), 'MockBean__property');
    });
    test('toBeanKey: bean + token + property', () {
      expect(
        toBeanKey(
          MockBean,
          token: 'token',
          property: 'property',
        ),
        'MockBean_token_property',
      );
    });
    test('get without token', () {
      expect(observableModel.getBean<MockBean>(), mockBean);
    });
    test('get with token', () {
      expect(observableModel.getBean<MockBean>(token), mockBeanWithToken);
    });
    test('remove bean with token', () {
      observableModel.removeBean<MockBean>(token: token);
      expect(observableModel.getBean<MockBean>(token), null);
      expect(observableModel.getBean<MockBean>(), mockBean);
    });
    test('clear map', () {
      observableModel.clearMap();
      expect(observableModel.getBean<MockBean>(), null);
    });
    test('put all beans', () {
      final other = MockBean();
      final beans = {
        toBeanKey(MockBean, token: token): other,
        toBeanKey(MockBean): mockBean
      };
      observableModel.putAllBeans(beans);
      expect(observableModel.getBean<MockBean>(), mockBean);
      expect(observableModel.getBean<MockBean>(token), other);
    });
  });
}
