import 'package:flutter/material.dart';
import 'package:flutter_gadgets/flutter_gadgets.dart';
import 'package:flutter_test/flutter_test.dart';

import '../utils.dart';

void main() {
  testWidgets('SubscriberWidget value', (tester) async {
    await tester.pumpWidget(AppGadget(
      beans: {toBeanKey(MockBean): MockBean()},
      child: SubscriberGadget<MockBean>(
        builder: (_, bean) {
          return Text(
            '${bean.state}',
            textDirection: TextDirection.ltr,
          );
        },
      ),
    ));
    expect(find.text('0'), findsOneWidget);

    await tester.pumpWidget(Container());
  });

  testWidgets('SubscriberWidget value changed with notify', (tester) async {
    final key = UniqueKey();
    await tester.pumpWidget(AppGadget(
      beans: {toBeanKey(MockBean): MockBean()},
      child: SubscriberGadget<MockBean>(
        builder: (_, bean) {
          return GestureDetector(
            onTap: () {
              final bean =
                  ServiceContainer.instance.observableModel.getBean<MockBean>();
              bean.increment();
              ServiceContainer.instance.observableModel.notifyFor(MockBean);
            },
            child: Text(
              '${bean.state}',
              key: key,
              textDirection: TextDirection.ltr,
            ),
          );
        },
      ),
    ));
    await tester.tap(find.byKey(key));
    expect(ServiceContainer.instance.observableModel.getBean<MockBean>().state, 1);
    await tester.pump();
    expect(find.text('1'), findsOneWidget);

    await tester.pumpWidget(Container());
  });

  testWidgets('SubscriberWidget value changed without notify', (tester) async {
    final key = UniqueKey();
    await tester.pumpWidget(AppGadget(
      beans: {toBeanKey(MockBean): MockBean()},
      child: SubscriberGadget<MockBean>(
        builder: (_, bean) {
          return GestureDetector(
            onTap: () {
              final bean =
                  ServiceContainer.instance.observableModel.getBean<MockBean>();
              bean.increment();
              ServiceContainer.instance.observableModel.notifyFor(MockBean);
            },
            child: Text(
              '${bean.state}',
              key: key,
              textDirection: TextDirection.ltr,
            ),
          );
        },
      ),
    ));
    final bean = ServiceContainer.instance.observableModel.getBean<MockBean>();
    bean.increment();
    expect(ServiceContainer.instance.observableModel.getBean<MockBean>().state, 1);
    await tester.pump();
    expect(find.text('1'), findsNothing);

    await tester.pumpWidget(Container());
  });
}
